# -*- encoding: utf-8 -*-
import sys

from github import GithubException

from suite_py.lib.handler.youtrack_handler import YoutrackHandler
from suite_py.lib.handler.github_handler import GithubHandler
from suite_py.lib.logger import Logger
from suite_py.lib.config import Config
from suite_py.lib.handler import git_handler as git
from suite_py.lib.handler import prompt_utils
from suite_py.commands import ask_review


youtrack = YoutrackHandler()
github = GithubHandler()
config = Config()
logger = Logger()


def entrypoint(project):
    branch_name = git.current_branch_name()

    if not git.remote_branch_exists(project, branch_name):
        logger.warning(f"Non ho trovato il branch {branch_name} su GitHub")
        if prompt_utils.ask_confirm("Vuoi commitare tutti i file e pusharlo?"):
            git.add(project)
            git.commit(project, "Initial commit")
            git.push(project, branch_name)
        else:
            logger.error("Per favore esegui un 'git push' manualmente")
            sys.exit(-1)

    youtrack_id = youtrack.get_card_from_name(branch_name)
    if youtrack_id:
        pulls = github.get_pr_from_branch(project, branch_name)
        if pulls.totalCount:
            pr = pulls[0]
            logger.info(
                f"Esiste una pull request su GitHub per il branch {branch_name}"
            )

            if prompt_utils.ask_confirm(
                "Vuoi modificare la description della pull request?"
            ):
                edit_pr(pr)
                sys.exit(0)

        else:
            create_pr(project, branch_name, youtrack_id)
    else:
        logger.warning(
            "Non sono riuscito a trovare una issue YouTrack nel nome del branch o la issue indicata non esiste"
        )
        if not prompt_utils.ask_confirm(
            "Vuoi collegare la pull request con una issue?"
        ):
            create_pr(project, branch_name)
        else:
            create_pr(project, branch_name, ask_for_id_card())


def edit_pr(pr):
    pr_body = ask_for_description(pr.body)
    pr.edit(body=pr_body)
    logger.info("Pull request modificata")


def create_pr(repo, branch_name, id_card=None):
    if id_card:
        logger.info(
            f"Creo una pull request sul progetto {repo} per il branch {branch_name} collegato con la card {id_card}"
        )
        link = youtrack.get_link(id_card)
        title = f"[{id_card}]: {youtrack.get_issue(id_card)['summary']}"
    else:
        logger.warning(
            f"Creo una pull request sul progetto {repo} per il branch {branch_name} SENZA collegamenti a YouTrack"
        )
        link = ""
        title = ask_for_title()

    base_branch = ask_for_base_branch()
    pr_body = ask_for_description()

    body = f"{link} \n\n {pr_body}"

    try:
        pr = github.create_pr(repo, branch_name, title, body, base_branch)
        logger.info(f"Pull request numero {pr.number} creata! {pr.html_url}")
    except GithubException as e:
        logger.error("Errore durante la richiesta a GitHub: ")
        logger.error(e.data["errors"][0])
        sys.exit(-1)

    if id_card:
        update_card(id_card, repo, pr.html_url)
        logger.info(f"Inserito link della pull request nella card {id_card}")

    if prompt_utils.ask_confirm("Vuoi inserire i reviewers?"):
        ask_review.entrypoint(repo, config.load()["user"]["captainhook_timeout"])


def ask_for_base_branch():
    branch = prompt_utils.ask_questions_input(
        "Inserisci il base branch della pull request: ", "master"
    )
    return branch


def ask_for_description(pr_body=""):
    input(
        "Inserisci la description della pull request. Premendo invio si aprira l'editor di default"
    )
    description = prompt_utils.ask_questions_editor(
        "Inserisci la description della PR: ", pr_body
    )
    if description == "":
        logger.warning("La descrizione della pull request non può essere vuota")
        return ask_for_description(pr_body)
    return description


def ask_for_id_card():
    id_card = prompt_utils.ask_questions_input(
        "Inserisci ID della card (ex: PRIMA-1234): "
    )
    if youtrack.validate_issue(id_card):
        return id_card
    logger.error("ID non esistente su YouTrack")
    return ask_for_id_card()


def update_card(id_card, repo, link):
    youtrack.comment(id_card, f"PR {repo} -> {link}")


def ask_for_title():
    title = prompt_utils.ask_questions_input("Inserisci il titolo della pull request: ")
    if title == "":
        logger.warning("Il titolo della pull request non può essere vuoto")
        return ask_for_title()
    return title



# -*- encoding: utf-8 -*-
import os
import re
import subprocess
from subprocess import CalledProcessError
import sys
import semver

from halo import Halo
from suite_py.lib.logger import Logger
from suite_py.lib.config import Config

logger = Logger()
config = Config()


def _get_path(repo):
    return os.path.join(config.load()["user"]["projects_home"], repo)


def clone(repo):
    subprocess.run(
        ["git", "clone", f"git@github.com:primait/{repo}.git"],
        cwd=config.load()["user"]["projects_home"],
        check=True,
    )


def checkout(repo, branch, new=False):
    try:
        if new:
            subprocess.run(
                ["git", "checkout", "-b", branch],
                cwd=_get_path(repo),
                check=True,
                stdout=subprocess.DEVNULL,
                stderr=subprocess.DEVNULL,
            )
        else:
            subprocess.run(
                ["git", "checkout", branch],
                cwd=_get_path(repo),
                check=True,
                stdout=subprocess.DEVNULL,
                stderr=subprocess.DEVNULL,
            )
            pull(repo, rebase=True)
    except (CalledProcessError) as e:
        if e.returncode == 128:
            checkout(repo, branch)  # caso checkout -b e branch gia' esistente
        elif e.returncode == 1:
            # caso checkout normale e branch non esistente
            checkout(repo, branch, new=True)
        else:
            logger.error(f"Errore eseguendo il comando: {e}")
            sys.exit(-1)


def commit(repo, commit_message="", dummy=False):
    try:
        if dummy:
            subprocess.run(
                ["git", "commit", "--allow-empty", "-m", commit_message],
                cwd=_get_path(repo),
                check=True,
            )
        else:
            subprocess.run(
                ["git", "commit", "-am", commit_message],
                cwd=_get_path(repo),
                check=True,
            )
    except (CalledProcessError) as e:
        logger.error(f"Errore eseguendo il comando: {e}")
        sys.exit(-1)


def add(repo):
    try:
        subprocess.run(
            ["git", "add", "."], cwd=_get_path(repo), check=True,
        )
    except (CalledProcessError) as e:
        logger.error(f"Errore eseguendo il comando: {e}")
        sys.exit(-1)


def push(repo, branch, remote="origin"):
    try:
        subprocess.run(["git", "push", remote, branch], cwd=_get_path(repo), check=True)
    except (CalledProcessError) as e:
        logger.error(f"Errore eseguendo il comando: {e}")
        sys.exit(-1)


def pull(repo, rebase=False):
    with Halo(text="Pulling...", spinner="dots", color="magenta"):
        try:
            if rebase:
                subprocess.run(
                    ["git", "pull", "--rebase"],
                    cwd=_get_path(repo),
                    check=True,
                    stdout=subprocess.DEVNULL,
                    stderr=subprocess.DEVNULL,
                )
            else:
                subprocess.run(
                    ["git", "pull"],
                    cwd=_get_path(repo),
                    check=True,
                    stdout=subprocess.DEVNULL,
                    stderr=subprocess.DEVNULL,
                )
        except (CalledProcessError) as e:
            logger.error(f"Errore eseguendo il comando: {e}")
            sys.exit(-1)


def fetch(repo, remote="origin"):
    with Halo(text="Fetching...", spinner="dots", color="magenta"):
        try:
            subprocess.run(
                ["git", "fetch", "--quiet"],
                cwd=_get_path(repo),
                check=True,
                stdout=subprocess.DEVNULL,
                stderr=subprocess.DEVNULL,
            )
            subprocess.run(
                ["git", "fetch", "-p", remote, "--quiet"],
                cwd=_get_path(repo),
                check=True,
                stdout=subprocess.DEVNULL,
                stderr=subprocess.DEVNULL,
            )
        except (CalledProcessError) as e:
            logger.error(f"Errore eseguendo il comando: {e}")
            sys.exit(-1)


def sync(repo):
    fetch(repo)
    checkout(repo, "master")
    pull(repo)


def check_repo_cloned(repo):
    if repo not in os.listdir(config.load()["user"]["projects_home"]):
        logger.warning(
            "Il progetto non e' presente nella tua home dei progetti, lo clono adesso.."
        )
        clone(repo)


def delete_remote_branch(repo, branch):
    try:
        push(repo, f":{branch}")
        return True
    except BaseException:
        return False


def get_username():
    try:
        output = subprocess.run(
            ["git", "config", "user.name"], check=True, stdout=subprocess.PIPE
        ).stdout.decode("utf-8")
        output = re.sub("[^A-Za-z]", "", output)
        return re.sub("\n", "", output)
    except CalledProcessError:
        return None


def get_last_tag(project):
    sync(project)
    return (
        subprocess.check_output(["git", "describe", "--abbrev=0", "--tags"])
        .decode("utf-8")
        .strip()
    )


def search_remote_branch(repo, regex):
    try:
        output = subprocess.run(
            ["git", "branch", "-r", "--list", regex],
            cwd=_get_path(repo),
            check=True,
            stdout=subprocess.PIPE,
        )

        return output.stdout.decode("utf-8").strip("\n").strip().strip("origin/")
    except CalledProcessError:
        return ""


def remote_branch_exists(repo, branch):
    try:
        fetch(repo)
        subprocess.run(
            ["git", "show-ref", "--quiet", f"refs/remotes/origin/{branch}"],
            cwd=_get_path(repo),
            check=True,
        )
        return True
    except CalledProcessError:
        return False


def local_branch_exists(repo, branch):
    try:
        subprocess.run(
            ["git", "show-ref", "--quiet", f"refs/heads/{branch}"],
            cwd=_get_path(repo),
            check=True,
        )
        return True
    except CalledProcessError:
        return False


def current_branch_name():
    try:
        output = subprocess.run(
            ["git", "rev-parse", "--abbrev-ref", "HEAD"],
            stdout=subprocess.PIPE,
            check=True,
        ).stdout.decode("utf-8")
        return re.sub("\n", "", output)
    except (CalledProcessError) as e:
        logger.error(f"Errore eseguendo il comando: {e}")
        sys.exit(-1)


def is_dirty(repo):
    try:
        subprocess.run(["git", "diff", "--quiet"], check=True)
        subprocess.run(["git", "diff", "HEAD", "--name-only", "--quiet"], check=True)
        return False
    except CalledProcessError:
        logger.error(
            f"Hai modifiche non committate su {repo} oppure non è un repository!"
        )
        sys.exit(-1)


def files_changed_between_commits(from_commit, to_commit):
    p = subprocess.Popen(
        ["git", "diff", from_commit, to_commit, "--name-only"], stdout=subprocess.PIPE
    )
    result = p.communicate()[0]
    return result.decode("utf-8").splitlines()


def migrations_found(files_changed):
    for file in files_changed:
        if "migration" in file.lower():
            return True
    return False


def get_last_tag_number(tags):
    for tag in tags:
        if is_semver(tag.name):
            return tag.name
    logger.warning("Nessun tag trovato, sto per pubblicare tag 0.1.0")
    return "0.1.0"


def is_semver(tag):
    try:
        return semver.parse(tag)
    except Exception:
        return None
